;; L3G4200D Digital Gyro
;;
;; Copyright 2012 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike to mikekohn.net
;;

.include "msp430x2xx.inc"

RAM equ 0x0200
CENTER_X equ RAM
CENTER_Y equ RAM+2
CENTER_Z equ RAM+4

;  r4 = X
;  r5 = sum(X)
;  r6 = Y
;  r7 = sum(Y)
;  r8 = Z
;  r9 = sum(Z)
; r10 = temperature (pointless)
; r11 = counter in interrupt
; r12 = main temp
; r13 = interrupt routine
; r14 = main function paramter 1
; r15 = interrupt function paramter 1

  org 0xc000
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; r13 points to which interrupt routine should be called
  mov.w #null_interrupt, r13

  ;; Set up stack pointer
  mov.w #0x0400, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

  ;; Set up output pins
  ;; P2.4 = /CS for SPI
  mov.b #0x00, &P1DIR
  mov.b #0x00, &P1OUT
  mov.b #0xe6, &P1SEL
  mov.b #0xe6, &P1SEL2
  mov.b #0x10, &P2DIR
  mov.b #0x10, &P2OUT
  mov.b #0x00, &P2SEL
  mov.b #0x00, &P2SEL2

  ;; Set up Timer
  mov.w #21000, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  ;; Setup UART
  mov.b #UCSSEL_2|UCSWRST, &UCA0CTL1
  mov.b #0, &UCA0CTL0
  mov.b #0xf8, &UCA0BR0
  mov.b #0x05, &UCA0BR1
  ;mov.b #0x20, &UCA0BR0
  ;mov.b #0x06, &UCA0BR1
  bic.b #UCSWRST, &UCA0CTL1

.if 0
  ;; Set up SPI USI
  mov.b #(USIPE7|USIPE6|USIPE5|USIMST|USIOE|USISWRST), &USICTL0
  ;mov.b #USICKPH, &USICTL1
  mov.b #0, &USICTL1
  mov.b #USIDIV_3|USISSEL_2|USICKPL, &USICKCTL ; div 128, SMCLK
  bic.b #USISWRST, &USICTL0      ; clear reset
.endif 

  ;; Set up SPI USCI
  mov.b #UCSSEL_3|UCSWRST, &UCB0CTL1
  ;mov.b #UCCKPH|UCCKPL|UCMSB|UCMST|UCSYNC, &UCB0CTL0
  mov.b #UCCKPL|UCMSB|UCMST|UCSYNC, &UCB0CTL0
  mov.b #4, &UCB0BR0
  mov.b #0, &UCB0BR1
  ;bic.b #UCLISTEN, &UCB0STAT
  bic.b #UCSWRST, &UCB0CTL1

  mov #0x2f, r14
delay_outer:
  mov #0xffff, r15
delay:
  dec r15
  jnz delay
  dec r14
  jnz delay_outer

  mov.w #0, r5
  mov.w #0, r7
  mov.w #0, r9
  mov.w #0, &CENTER_X
  mov.w #0, &CENTER_Y
  mov.w #0, &CENTER_Z
  mov.w #0, r11

  ;; Okay, I can be interrupted now
  eint

  ;; Read WHOAMI
  bic.b #0x10, &P2OUT
  mov.b #0x8f, r15
  call #spi_send_char
  call #spi_get_char
  bis.b #0x10, &P2OUT

  ;; CTRL_REG1 - Take this chip out of power down mode
  bic.b #0x10, &P2OUT
  mov.b #0x20, r15
  call #spi_send_char
  mov.b #0xff, r15       ; dr=3, bw=3, pd=1, Zen=1, Yen=1, Xen=1
  call #spi_send_char
  bis.b #0x10, &P2OUT

  ;; CTRL_REG2 - High pass filter mode
  bic.b #0x10, &P2OUT
  mov.b #0x21, r15
  call #spi_send_char
  mov.b #0x15, r15
  call #spi_send_char
  bis.b #0x10, &P2OUT

  ;; CTRL_REG4
  bic.b #0x10, &P2OUT
  mov.b #0x23, r15
  call #spi_send_char
  mov.b #0x70, r15
  call #spi_send_char
  bis.b #0x10, &P2OUT

  ;; CTRL_REG5 - Filter data
  bic.b #0x10, &P2OUT
  mov.b #0x24, r15
  call #spi_send_char
  mov.b #0x52, r15
  call #spi_send_char
  bis.b #0x10, &P2OUT

  ;; Let's try stream mode
  bic.b #0x10, &P2OUT
  mov.b #0x2e, r15
  call #spi_send_char
  mov.b #0x40, r15
  call #spi_send_char
  bis.b #0x10, &P2OUT

  mov.w #calibrate_gyro_interrupt, r13

main:
  mov.w r4, r14
  call #uart_send_char
  mov.w r5, r14
  call #uart_send_char
  mov.w r6, r14
  call #uart_send_char
  mov.w r7, r14
  call #uart_send_char
  mov.w #0x7fff, r14
  call #uart_send_char
  mov.w #0x0002, r12
delay_main_outer:
  mov.w #0xffff, r14
delay_main:
  dec r14
  jnz delay_main
  dec r12
  jnz delay_main_outer
  jmp main

uart_send_char:
  mov.w r14, r12
  swpb r12
  mov.b r14, &UCA0TXBUF
wait_lsb_tx:
  bit.b #UCA0TXIFG, &IFG2
  jz wait_lsb_tx

  mov.b r12, &UCA0TXBUF
wait_msb_tx:
  bit.b #UCA0TXIFG, &IFG2
  jz wait_msb_tx
  ret

; spi_send_char(r15)
spi_send_char:
  mov.b r15, &UCB0TXBUF
wait_spi_send_char:
  bit.b #UCB0RXIFG, &IFG2
  jz wait_spi_send_char
  mov.b &UCB0RXBUF, r15         ; reset UCB0RXIFG
  ret

.if 0
; spi_send_char(r15)
spi_send_char:
  mov.b r15, &USISRL
  mov.b #8, &USICNT
wait_spi_read_write:
  bit.b #USIIFG, &USICTL1
  jz wait_spi_read_write
  mov.b &USISRL, r15
  ret
.endif

; spi_get_char()
spi_get_char:
  mov.b #0xff, &UCB0TXBUF
wait_spi_get_char:
  bit.b #UCB0RXIFG, &IFG2
  jz wait_spi_get_char
  mov.b &UCB0RXBUF, r15
  ret

.if 0
; spi_get_char()
spi_get_char:
  mov.b #0xff, &USISRL
  mov.b #8, &USICNT
wait_spi_get_char:
  bit.b #USIIFG, &USICTL1
  jz wait_spi_get_char
  mov.b &USISRL, r15
  ret
.endif

timer_interrupt:
  br r13

null_interrupt:
  reti

read_gyro_interrupt:
  ;; READ X
  bic.b #0x10, &P2OUT
  mov.b #0xe8, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r4
  call #spi_get_char
  swpb r15
  bis.w r15, r4
  bis.b #0x10, &P2OUT
  ;sub.w &CENTER_X, r4
  rra.w r4
  rra.w r4
  rra.w r4
  rra.w r4
  rra.w r4
  add.w r4, r5

  ;; READ Y
  bic.b #0x10, &P2OUT
  mov.b #0xea, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r6
  call #spi_get_char
  swpb r15
  bis.w r15, r6
  bis.b #0x10, &P2OUT
  ;sub.w &CENTER_Y, r6
  rra.w r6
  rra.w r6
  rra.w r6
  rra.w r6
  rra.w r6
  add.w r6, r7

  ;; READ Z
  bic.b #0x10, &P2OUT
  mov.b #0xec, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r8
  call #spi_get_char
  swpb r15
  bis.w r15, r8
  bis.b #0x10, &P2OUT
  ;sub.w &CENTER_Z, r8
  add.w r8, r9

.if 0
  ;; READ temp
  bic.b #0x10, &P2OUT
  mov.b #0xe6, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r10
  ;call #spi_get_char
  ;swpb r15
  ;bis.w r15, r10
  bis.b #0x10, &P2OUT
.endif
  reti

calibrate_gyro_interrupt:
  ;; READ X
  bic.b #0x10, &P2OUT
  mov.b #0xe8, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r4
  call #spi_get_char
  swpb r15
  bis.w r15, r4
  add.w r4, &CENTER_X

  ;; READ Y
  bic.b #0x10, &P2OUT
  mov.b #0xea, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r6
  call #spi_get_char
  swpb r15
  bis.w r15, r6
  add.w r6, &CENTER_Y

  ;; READ Z
  bic.b #0x10, &P2OUT
  mov.b #0xec, r15
  call #spi_send_char
  call #spi_get_char
  mov.b r15, r8
  call #spi_get_char
  swpb r15
  bis.w r15, r8
  add.w r8, &CENTER_Z

  add.w #1, r11
  cmp.w #128, r11
  jnz not_done_calibrating

  ;swpb &CENTER_X
  ;swpb &CENTER_Y
  ;swpb &CENTER_Z
  ;and.w #0xff, &CENTER_X
  ;and.w #0xff, &CENTER_Y
  ;and.w #0xff, &CENTER_Z
  ;sxt &CENTER_X
  ;sxt &CENTER_Y
  ;sxt &CENTER_Z

  rra.w &CENTER_X
  rra.w &CENTER_X
  rra.w &CENTER_X
  rra.w &CENTER_X

  rra.w &CENTER_X
  rra.w &CENTER_X
  rra.w &CENTER_X

  rra.w &CENTER_Y
  rra.w &CENTER_Y
  rra.w &CENTER_Y
  rra.w &CENTER_Y

  rra.w &CENTER_Y
  rra.w &CENTER_Y
  rra.w &CENTER_Y

  mov.w #read_gyro_interrupt, r13
not_done_calibrating:
  reti

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset


